from typing import Protocol

import jax
from jaxtyping import Float

type X = Float[jax.Array, " N"]
type Y = Float[jax.Array, ""]


class ProblemProtocol(Protocol):
    def fun(self, x: X, /, *args, **kwargs) -> Y: ...
    def jac(self, x: X, /, *args, **kwargs) -> X: ...
    def hessp(self, x: X, p: X, /, *args, **kwargs) -> X: ...
    def hess_diag(self, x: X, /, *args, **kwargs) -> X: ...
    def hess_quad(self, x: X, p: X, /, *args, **kwargs) -> Y: ...
    def fun_and_jac(self, x: X, /, *args, **kwargs) -> tuple[Y, X]: ...
    def jac_and_hess_diag(self, x: X, /, *args, **kwargs) -> tuple[X, X]: ...
